#!/usr/bin/perl -w

print "<?xml version=\"1.0\"?>\n";
print "<nodeinfo>\n";
print xmlencode("hostname",`hostname`),"\n";
print xmlencode("date",`date`),"\n";
print xmlencode("time",time()),"\n";
print xmlencodeFile("redhat","/etc/redhat-release"),"\n";
print xmlencode("linux",`uname -r`),"\n";
print xmlencodeFile("cpuinfo","/proc/cpuinfo"),"\n";
print xmlencodeFile("meminfo","/proc/meminfo"),"\n";
print xmlencode("uptime",`uptime`),"\n";
print xmlencode("chkconfig",`/sbin/chkconfig --list | grep on`),"\n";
print xmlencode("etchosts",`grep -v ^# /etc/hosts`),"\n";
print xmlencode("etcresolvconf",`grep -v ^# /etc/resolv.conf`),"\n";
my $domainname = `domainname`; chop $domainname;
if ($domainname ne "(none)")
  {
    print xmlencode("domainname",`domainname`),"\n";
    print xmlencode("etcypconf",`grep -v ^# /etc/yp.conf`),"\n";
    print xmlencode("ypwhich",`ypwhich`),"\n";
    my $ypserv = system "/sbin/chkconfig ypserv";
    if ($ypserv == 0)
      {
        print xmlencode("ypserv",`/sbin/pidof /usr/sbin/ypserv`),"\n";
      }
  }
print xmlencode("ifconfig",`/sbin/ifconfig`),"\n";
print xmlencodeFile("ifcfgeth0","/etc/sysconfig/network-scripts/ifcfg-eth0"),"\n";
print xmlencode("miitool",`/sbin/mii-tool 2>/dev/null`),"\n";
print xmlencodeFile("printcap","/etc/printcap"),"\n";
if (-x "/usr/bin/ntpstat")
  {
    print xmlencode("ntpstat",`/usr/bin/ntpstat`),"\n";
  }
elsif (-x "/usr/sbin/ntptrace")
  {
    print xmlencode("ntp",`/usr/sbin/ntptrace`),"\n";
  }
print xmlencode("mailspool",`/bin/ls -l /var/spool/mail`),"\n";
print xmlencode("ide",encodeIde()),"\n";
print encodeSmart();
print xmlencodeFile("scsi","/proc/scsi/scsi"),"\n";
print xmlencodeFile("etcnodeinfo","/etc/nodeinfo"),"\n";
print xmlencode("lspci",`/sbin/lspci`),"\n";
print xmlencodeFile("rootforward","/root/.forward"),"\n";
print xmlencodeFile("hostsallow","/etc/hosts.allow"),"\n";
print xmlencodeFile("hostsdeny", "/etc/hosts.deny"),"\n";
print xmlencodeFile("securenets","/var/yp/securenets"),"\n";
print xmlencodeFile("mdstat","/proc/mdstat"),"\n";
print xmlencodeFile("diskscrub","/var/log/diskscrub.last.txt"),"\n";
print xmlencode("dmidecode",`/usr/sbin/dmidecode`),"\n" if (-r "/dev/mem") && (-x "/usr/sbin/dmidecode");
print xmlencode("rpmnew",`/usr/bin/locate rpmnew`),"\n";
print xmlencode("rpmsave",`/usr/bin/locate rpmsave`),"\n";
print xmlencode("rpms",`rpm -q -a | sort -u`),"\n";
print "</nodeinfo>\n";

exit 0;

sub xmlencodeFile
  {
    my $tag  = shift @_;
    my $file = shift @_;
    return if ! -e $file;
    open(TMPIN,$file) || return "";
    my $contents = join("",<TMPIN>);
    close TMPIN;
    $contents =~ s/\n$//;
    return xmlencode($tag,$contents);
  }

sub xmlencode
  {
    my $tag = shift @_;
    my $contents = join("",@_);
    $contents =~ s/\n$//;
    $contents =~ s/\</&lt;/m;
    $contents =~ s/\>/&gt;/m;
    return "<$tag>$contents</$tag>";
  }

sub encodeIde24
  {
    my $ret = "";
    foreach my $drive (split(/\n/,`/bin/ls -1 /proc/ide`))
      {
	next if ! -r "/proc/ide/$drive/capacity";
	my $cap   = `cat /proc/ide/$drive/capacity`; chop $cap;
	my $model = `cat /proc/ide/$drive/model`; chop $model;
	my $cache = "none";
	if (-r "/proc/ide/$drive/cache")
	  {
	    $cache = `cat /proc/ide/$drive/cache`; chop $cache;
	  }
	$ret .= "$drive:$cap:$cache:$model\n";
      }
    return $ret;
  }

sub encodeIde26
  {
    my $ret = "";
    foreach my $drive (split(/\n/,`/bin/ls -1 /sys/block`))
      {
	next if ! -r "/sys/block/$drive/size";
        next if ($drive =~ /^ram/);
        next if ($drive =~ /^loop/);
	my $cap   = xreadline("/sys/block/$drive/size");
        $cap = 0 if ! defined $cap;
	my $model = xreadline("/sys/block/$drive/device/model");
	$model = xreadline("/proc/ide/$drive/model") if ! defined $model;
        $model = "(unknown)" if ! defined $model;
        my $removable = xreadline("/sys/block/$drive/removable");
        $cap = 0 if $removable;
	my $cache = xreadline("/proc/ide/$drive/cache");
        $cache = "none" if ! defined $cache;
        next if ($removable==0)&&($cap==0);
	$ret .= "$drive:$cap:$cache:$model";
        $ret .= " (removable)" if $removable;
        $ret .= "\n";
      }
    return $ret;
  }

sub encodeIde
  {
    return encodeIde26() if -r "/sys/block";
    return encodeIde24();
  }

sub encodeSmart
  {
    my $smartctl = "/usr/sbin/smartctl";

    return "" if ! -x $smartctl;

    my $xml = "";

    $xml .= "<smart>\n";

    if (-d "/sys/block")
      {
        opendir(DIR,"/sys/block");
        while (my $f = readdir(DIR))
          {
            my $dev = "/dev/$f";
            next unless -r $dev;
            next if -d $dev;
            #print "$dev\n";

            my $data = getsmart($smartctl,$dev);
            next unless $data;

            #print "\n\n\n$dev\n$data\n\n\n";
            #die "Here!";

            $xml .= "<data dev=\"$dev\">\n".$data."</data>\n";
          }
        closedir(DIR);

        #die "Here!";
      }
    else
      {
        foreach my $dev ("/dev/hda", "/dev/hdb", "/dev/hdc", "/dev/hdd")
          {
            next if ! -r $dev;

            my $data = getsmart($smartctl,$dev);
            next unless $data;
            
            $xml .= "<data dev=\"$dev\">\n".$data."</data>\n";
          }
      }

    if (`/sbin/lspci` =~ /3ware/)
      {
        my $dev = "/dev/twe0";

        print "Please create 3ware control device: mknod /dev/twe0 c 254 0\n" unless -r $dev;

        for (my $unit=0; $unit<=16; $unit++)
          {
            my $data = `$smartctl --device=3ware,$unit -a $dev 2>&1`;
            
            next if ! ($data =~ /SMART support is: Available/);
            
            $xml .= "<data dev=\"3ware/$unit\">\n".$data."</data>\n";
          }
      }
    
    $xml .= "</smart>\n";

    return $xml;
  }

sub getsmart
  {
    my $smartctl = shift @_;
    my $dev = shift @_;

    my $data = `$smartctl -a $dev 2>&1`;

    return if ($data =~ /please specify device type/); # smartctl does not know what this is.

    if ($data =~ /SATA disks accessed/)
      {
        $smartctl .= " -d ata";
        $data = `$smartctl -a $dev 2>&1`;
      }

    if ($data =~ /Use option -e to enable/)
      {
        system "$smartctl -e $dev >/dev/null 2>&1";
        $data = `$smartctl -a $dev 2>&1`;
      }
    if ($data =~ /Use option -s with argument 'on' to enable it/)
      {
        system "$smartctl -s on $dev >/dev/null 2>&1";
        $data = `$smartctl -a $dev 2>&1`;
      }
    
    # check return values from RHL8.0-9 smartctl
    return if $data =~ /No such device or address/; # no device
    return if $data =~ /Read-only file system/; # probably a CDROM
    return if $data =~ /Drive does not support/; # probably a CDROM
    return if $data =~ /SMART support is: Unavailable/; # probably a CDROM
    return if $data =~ /A mandatory SMART command failed/; # sorry, no SMART support
    
    # check return values from Fedora-1 smartctl
    if ($data =~ /-T permissive/)
      {
        $data = `$smartctl -T permissive -a $dev 2>&1`;
      }
    
    return if $data =~ /No such device/; # no device
    return if $data =~ /SMART support is: Unavailable/; # CDROM

    return $data;
}

sub xreadfile
  {
    my $filename = shift @_;
    open (TMPIN,$filename) || return;
    my $v = <TMPIN>;
    close TMPIN;
    return $v;
  }

sub xreadline
  {
    my $val = xreadfile(shift @_);
    return if ! defined $val;
    $val =~ s/\n+$//;
    return $val;
  }

#end file
